@@ -150,7 +150,7 @@ class Ecdsa:
     :rtype: dict
     """
     S_i = next(filter(lambda S_i: not 'j' in S_i, S))
-    ntilde, h1, h2 = generate_pre_params()
+    ntilde, h1, h2 = generate_ntilde()
     signers = {
       S_i['i']: {
         'i': S_i['i'],
@@ -197,28 +197,34 @@ class Ecdsa:
     if not 'ntilde' in S_i:
       S_i = self.sign_challenge(S)[S_i['i']]
     pka = phe.PaillierPublicKey(S_i['p'] * S_i['q'])
-    # Select $k_i, \gamma_i \in_R Z_q$.
-    k = self.curve.scalar_random()
+    if 'k' in S_i:
+      k = S_i['k']
+      gamma = S_i['gamma']
+      w = S_i['w']
+    else:
+      # Select $k_i, \gamma_i \in_R Z_q$.
+      k = self.curve.scalar_random()
+      gamma = self.curve.scalar_random()
+      # Map $x_i$ in $(n,t)$ to $w_i$ in $(t,t)$.
+      d = reduce(
+        lambda acc, S_j: self.curve.scalar_mul(
+          self.curve.scalar_sub(S_j['j'], S_i['i']),
+          acc,
+        ),
+        [S_i['i']] + list(filter(lambda S_j: 'j' in S_j, S)),
+      )
+      w = reduce(
+        self.curve.scalar_mul,
+        [S_i['x'], self.curve.scalar_invert(d)],
+        reduce(
+          lambda acc, S_j: self.curve.scalar_mul(S_j['j'], acc),
+          [S_i['i']] + list(filter(lambda S_j: 'j' in S_j, S)),
+        ),
+      )
+    # Encrypt k.
     rk = get_random_coprime_to(pka.n)
     ck = pka.encrypt(k, r_value=rk).ciphertext(False)
-    gamma = self.curve.scalar_random()
-    # Map $x_i$ in $(n,t)$ to $w_i$ in $(t,t)$.
-    d = reduce(
-      lambda acc, S_j: self.curve.scalar_mul(
-        self.curve.scalar_sub(S_j['j'], S_i['i']),
-        acc,
-      ),
-      [S_i['i']] + list(filter(lambda S_j: 'j' in S_j, S)),
-    )
-    w = reduce(
-      self.curve.scalar_mul,
-      [S_i['x'], self.curve.scalar_invert(d)],
-      reduce(
-        lambda acc, S_j: self.curve.scalar_mul(S_j['j'], acc),
-        [S_i['i']] + list(filter(lambda S_j: 'j' in S_j, S)),
-      ),
-    )
-    # Generate pre params.
+    # Get ntilde.
     ntildea, h1a, h2a = S_i['ntilde'], S_i['h1'], S_i['h2']
     signers = {
       S_i['i']: {
@@ -231,7 +237,6 @@ class Ecdsa:
         'h2': h2a,
         'k': k,
         'ck': ck,
-        'rk': rk,
         'w': w,
         'gamma': gamma,
       },
@@ -303,11 +308,16 @@ class Ecdsa:
 
     :rtype: dict
     """
-    S_i, S_j = None, None
+    S_i, S_j, k_share = None, None, None
     S_i = next(filter(lambda S_i: not 'j' in S_i, S))
-    if 'x' in S_i:
-      S_i = self.sign_share(S)[S_i['i']]
-    S_j = next(filter(lambda S_j: 'j' in S_j, S))
+    k_shares = list(filter(lambda S_j: 'j' in S_j and S_j['j'] == S_i['i'], S))
+    S_j = next(filter(lambda S_j: 'j' in S_j and S_j['i'] == S_i['i'], S))
+    if 'x' in S_i or len(k_shares) == 0 and not 'alpha' in S_j:
+      shares = self.sign_share(S)
+      S_i = shares[S_i['i']]
+      k_share = shares[next(filter(lambda i: 'j' in shares[i], shares))]
+    elif len(k_shares) > 0:
+      k_share = k_shares[0]
     assert S_i['i'] == S_j['i']
     S_i = S_i.copy()
     S_j = S_j.copy()
@@ -489,30 +499,23 @@ class Ecdsa:
         del S_i['h1']
         del S_i['h2']
         del S_i['ck']
-        del S_i['rk']
         del S_j['n']
         del S_j['ntilde']
         del S_j['h1']
         del S_j['h2']
         del S_j['k']
       else:
-        pkb = phe.PaillierPublicKey(S_i['p'] * S_i['q'])
-        k = S_i['k']
-        ck = S_i['ck']
-        rk = S_i['rk']
-        z, u, w, s, s1, s2 = prove_range(self.curve, pkb, ck, ntildea, h1a, h2a, k, rk)
-        S_j['n'] = pkb.n
         S_j['ntilde'] = ntildeb
         S_j['h1'] = h1b
         S_j['h2'] = h2b
-        S_j['k'] = ck
-        S_j['z'] = z
-        S_j['u'] = u
-        S_j['w'] = w
-        S_j['s'] = s
-        S_j['s1'] = s1
-        S_j['s2'] = s2
-        del S_i['rk']
+        S_j['n'] = k_share['n']
+        S_j['k'] = k_share['k']
+        S_j['z'] = k_share['z']
+        S_j['u'] = k_share['u']
+        S_j['w'] = k_share['w']
+        S_j['s'] = k_share['s']
+        S_j['s1'] = k_share['s1']
+        S_j['s2'] = k_share['s2']
     if not 'alpha' in S_j and not 'k' in S_j:
       S_i['j'] = S_j['j']
       del S_i['ntilde']
@@ -697,15 +700,13 @@ def to_bytes(x, byteorder):
   byte_length = max(1, (x.bit_length() + 7) // 8)
   return x.to_bytes(byte_length, byteorder)
 
-def generate_pre_params():
+def generate_ntilde(ntilde=None, f1=None, f2=None):
   pk, _ = phe.generate_paillier_keypair(n_length=MODULUS_BITS)
   ntilde = pk.n
-  pk, _ = phe.generate_paillier_keypair(n_length=MODULUS_BITS)
   f1 = get_random_coprime_to(ntilde)
-  alpha = get_random_coprime_to(ntilde)
-  beta = pow(alpha, -1, pk.n)
+  f2 = get_random_coprime_to(ntilde)
   h1 = pow(f1, 2, ntilde)
-  h2 = pow(h1, alpha, ntilde)
+  h2 = pow(f2, 2, ntilde)
   return (ntilde, h1, h2)
 
 def prove_range(curve, pk, c, ntilde, h1, h2, m, r):
@@ -796,12 +797,10 @@ def prove_range_wc(curve, pk, ntilde, h1, h2, c1, c2, x, y, r, X):
     hash.update(b'\x0d\x00\x00\x00\x00\x00\x00\x00' + b'$'.join([
       to_bytes(pk.n, 'big'),
       to_bytes(pk.g, 'big'),
-      to_bytes(int(X.x()), 'big'),
-      to_bytes(int(X.y()), 'big'),
+      X.to_bytes('compressed'),
       to_bytes(c1, 'big'),
       to_bytes(c2, 'big'),
-      to_bytes(int(u.x()), 'big'),
-      to_bytes(int(u.y()), 'big'),
+      u.to_bytes('compressed'),
       to_bytes(z, 'big'),
       to_bytes(zprm, 'big'),
       to_bytes(t, 'big'),
@@ -840,12 +839,10 @@ def verify_range_proof_wc(curve, pk, ntilde, h1, h2, z, zprm, t, v, w, s, s1, s2
     hash.update(b'\x0d\x00\x00\x00\x00\x00\x00\x00' + b'$'.join([
       to_bytes(pk.n, 'big'),
       to_bytes(pk.g, 'big'),
-      to_bytes(int(X.x()), 'big'),
-      to_bytes(int(X.y()), 'big'),
+      X.to_bytes('compressed'),
       to_bytes(c1, 'big'),
       to_bytes(c2, 'big'),
-      to_bytes(int(u.x()), 'big'),
-      to_bytes(int(u.y()), 'big'),
+      u.to_bytes('compressed'),
       to_bytes(z, 'big'),
       to_bytes(zprm, 'big'),
       to_bytes(t, 'big'),
